<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>upload@cascadis</title></head>
<body>
<form method="POST" enctype="multipart/form-data">
    <label for="file">{{ label or "Choose a file to upload" }}</label>
    <input type="file" id="file" name="file">
    <button>Submit</button>
</form>
</body>
</html>
