<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>login@cascadis</title></head>
<body>
<form method="POST">
    <label for="username">Username</label>
    <input type="text" id="username" name="username" value="{{ username }}">
    <label for="password">Password</label>
    <input type="password" id="password" name="password" value="{{ password }}">
    <button>Submit</button>
</form>
</body>
</html>
